##-------------------------------------------------------------
## $Id$
package Tax::Canada;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT_OK = qw(
    sales_tax
    pst
    gst
    hst
);
$VERSION = '0.01';

##-------------------------------------------------------------
use vars qw(%PST $GST $HST @HST_prov %Equiv);
$GST=0.07;
$HST=0.15;
%PST=(
    'AB'=>{pst=>0.00, },
    'BC'=>{pst=>0.07, },
    'MB'=>{pst=>0.07, },
    'NB'=>{hst=>1, },
    'NF'=>{hst=>1, },
    'NS'=>{hst=>1, },
    'ON'=>{pst=>0.08, },
    'PE'=>{pst=>0.10, piggyback=>1},
    'QC'=>{pst=>0.075, piggyback=>1, no_book=>1},
    'SK'=>{pst=>0.07},
    'YT'=>{pst=>0.00},
    'NT'=>{pst=>0.00},
    'CA'=>{},
);

@HST_prov=grep {$PST{$_}->{hst}} keys %PST;

%Equiv=(
    'Alberta'               =>'AB',
    'British Columbia'      =>'BC',
    'Colombie Britannique'  =>'BC',
    'Manitoba'              =>'MB',
    'New Brunswick'         =>'NB',
    'Nouveau-Brunswick'     =>'NB',
    'Newfoundland'          =>'NF',
    'Terre-Neuve'           =>'NF',
    'Nouvelle-Ecosse'       =>'NS',
    "Nouvelle-\Xc9cosse"    =>'NS',
    'Nova Scotia'           =>'NS',
    'Ontario'               =>'ON',
    'Ile-du-Prince-Edouard' =>'PE',
    "\xcele-du-Prince-\xC9douard" =>'PE',
    'Prince Edward Island'  =>'PE',
    'PEI'                   =>'PE',
    'Quebec'                =>'QC',
    "Qu\xe9bec"             =>'QC',
    'Saskatchewan'          =>'SK',
    'Yukon'                 =>'YT',
    'NWT'                   =>'NT',
    'TNW'                   =>'NT',
    'Hors Canada'           =>'HC',
    'Outside Canada'        =>'HC',
    'Canada'                =>'CA',
);


##-------------------------------------------------------------
sub sales_tax
{
    my($prov, $amount, $book)=@_;
    $prov||='HC';
    $prov=$Equiv{$prov} if(exists $Equiv{$prov});

    return (0,0,0) if $prov eq 'HC';

    my $total=$amount+$book;
    my $tps=0;
    my $pst=0;
    my $tvh=0;

    $tps=round($total*$GST, 0.01);
    return ($tps, 0, 0) if $prov eq 'CA';

    my $tt=$PST{$prov};

    if($tt->{piggyback} and $tt->{no_book})
    {   $pst=round(($amount + $amount*$GST)*$tt->{pst}, 0.01);
    } elsif($tt->{piggyback})
    {   $pst=round(($total+$tps)*$tt->{pst}, 0.01);
    } elsif($tt->{no_books})
    {   $pst=round($amount*$tt->{pst}, 0.01);
    } elsif($tt->{hst})
    {   $tvh=round($amount*$HST + $book*$GST, 0.01);
        $tps=0;
    } else
    {   $pst=round($total*$tt->{pst}, 0.01);
    }

    return ($tps, $pst, $tvh);
}

sub gst { (sales_tax(@_))[0]; }
sub pst { (sales_tax(@_))[1]; }
sub hst { (sales_tax(@_))[2]; }


##-------------------------------------------------------------
sub which_province
{
    my($client, @merchant)=@_;
    @merchant=map {exists $Equiv{$_} ? $Equiv{$_} : $_} @merchant;
    $client=$Equiv{$client} if(exists $Equiv{$client});

    return $client if grep {$_ eq $client} @merchant, 'HC', @HST_prov;
    return 'HC' unless exists $PST{$client};
    return 'CA';
}


##########################################
## round a value to a given preccesion
## $num=round($val, $precision);
sub round
{
    my($val, $prec)=@_;
    $prec=1 if (not $prec);
#     $prec=1/$prec if ($prec>1);       # this makes 100 and .01 equivilent
    return int(($val/$prec)+0.5)*$prec;
}

1;
__END__

=head1 NAME

Tax::Canada - Calculate Sales taxes for Canadian provinces.


=head1 SYNOPSIS

  use Tax::Canada;
  my($gst, $pst, $hst)=Tax::Canada::sales_tax('QC', 100.00);

  use Tax::Canada qw(sales_tax);
  $hardware=100;
  $books=45.69;
  my($gst, $pst, $hst)=sales_tax('QC', $hardware, $books);


=head1 DESCRIPTION

=head2 sales_tax

    ($gst, $pst, $hst)=$Tax::Canada::sales_tax($province, $non_book_dollar_amount, $book_dollar_amount);

Calculates the provincial sales tax (pst), harmonised sales taxes (hst)
as well as the goods and services tax (gst).  C<$province> is the province the
client resides in.  If the client does not reside in a province the merchant
has a office in, and the client does not reside in the provinces using HST,
C<$province> should be 'CA'.  If the client does not reside in Canada, the
province should be 'HC'.  See L<"which_province"> for more details.

C<$non_book_dollar_amount>,
C<$book_dollar_amount>: These are the purchase totals of non-book and
book items respectivly. They need to be calculated seperately because
many provinces do not charge sales tax on books.  Shipping and handling on
books should books must be included in C<$non_book_dollar_amount>.

=head2 pst, gst, hst

    $gst=Tax::Canada::gst($province, $non_book_dollar_amount, $book_dollar_amount);
    $pst=Tax::Canada::pst($province, $non_book_dollar_amount, $book_dollar_amount);
    $hst=Tax::Canada::hst($province, $non_book_dollar_amount, $book_dollar_amount);

=head2 which_province

    $prov=Tax::Canada::which_province($client, @merchant);

"Normalises" the client's province.  C<$client> is the province
of residence of the client.  C<@merchant> is a list of all provinces
that the merchant operates in.

=head2 About provinces

Provinces are the following two letter abbreviations:

    AB BC MB NB NF NS ON PE QC SK YT NT

C<%Tax::Canada::Equiv> defines many equivalancies, including full names
in English and French, with and without accents.  All routines use this
hash to turn provinces into their corresponding abbreviation.

=head1 AUTHOR

Philip Gwyn (gwynp@artware.qc.ca)

=head1 SEE ALSO

perl(1), a real accountant to make sure everything is kosher.

=head1 DISCLAIMER

While this module is correct to best of my knowledge, I make no warranty
as to the verasity, applicability or acuracy of the information contained.
In particular, sales taxes may be changed from time to time.  As always, you
should consult a financial professional.

Use or misuse of this module will implicitly absolve the Author of all
legal responisibility.

=cut

